@@ -1,41 +1,21 @@
 /*
- *		  mewencode, mewdecode, and mewcat
+ * mewencode, mewdecode, and mewcat --- MIME encoding for Mew
  *
- *	      Copyright (C) 1994-1997  Kazuhiko Yamamoto
- *
- * This software conforms GNU GENERAL PUBLIC LICENSE Version 2.
- * The modification copyright also conforms GNU GENERAL PUBLIC LICENSE
- * Version 2.
- *
- * Author:  Kazuhiko Yamamoto <kazu@is.aist-nara.ac.jp>
- * Created: December  8, 1994
- * Revised: February 27, 1995
+ * Author:  Kazu Yamamoto <Kazu@Mew.org>
+ * Created: Dec  8, 1994
+ * Revised: Nov 13, 1997
  *
+ * Code:
  */
 
-/*
- ****************************************************************
- Copyright (c) 1991 Bell Communications Research, Inc. (Bellcore)
- 
- Permission to use, copy, modify, and distribute this material 
- for any purpose and without fee is hereby granted, provided 
- that the above copyright notice and this permission notice 
- appear in all copies, and that the name of Bellcore not be 
- used in advertising or publicity pertaining to this 
- material without the specific, prior written permission 
- of an authorized representative of Bellcore.  BELLCORE 
- MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY 
- OF THIS MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", 
- WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.
- ****************************************************************
- */
-
-static char version_message[] =
-	"mewencode version 0.10 970304 Kazuhiko Yamamoto";
+static char version_message[] =	"mewencode version 0.13 971113 Kazu Yamamoto";
 
 #include "getopt.h"
 #include <stdio.h>
 #include <string.h>
+#if defined(_WIN32) || defined(OS2)
+#include <fcntl.h>
+#endif
 
 #define ENCODE_NAME "mewencode"
 #define DECODE_NAME "mewdecode"
@@ -70,17 +50,12 @@ int Outcr  = OFF;
 int Outeof = OFF;
 
 /*
- * System dependent
+ * "b" is ignored on UNIX.
  */
-#ifdef OS2
 #define FDREAD	"rb"
 #define FDWRITE	"wb"
+
 #define STRCMP(str1, str2) strncmp(str1, str2, sizeof(str2) - 1)
-#else /* OS2 */
-#define FDREAD	"r"
-#define FDWRITE	"w"
-#define STRCMP(str1, str2) strcmp(str1, str2)
-#endif /* OS2 */
 
 /*
  * long name convention for option
@@ -101,7 +76,7 @@ struct option longopts [] = {
 };
 
 
-usage(progname)
+void usage(progname)
 	char* progname;
 {
 	fprintf(stderr, "usage: %s [-e|-d|-s] [-b|-q|-g] [-l length] [-t] [infile [outfile]]\n", progname);
@@ -134,7 +109,7 @@ char *help_message[] = {
 	0
 };
 
-help(progname)
+void help(progname)
 	char *progname;
 {
 	char **p = help_message;
@@ -145,7 +120,7 @@ help(progname)
 	while (*p) fprintf(stderr, "%s\n", *p++);
 }
 
-version(progname)
+void version(progname)
 	char *progname;
 {
 	fprintf(stderr, "version of %s: %s\n", progname, version_message);
@@ -156,7 +131,7 @@ version(progname)
  */
 
 char *
-basename(filename)
+mewbasename(filename)
 	char *filename;
 {
 	char *p;
@@ -300,7 +275,7 @@ static char base64[] =
 
 #define PADDING '='
 
-base256to64 (c1, c2, c3, padding, outfile)
+void base256to64 (c1, c2, c3, padding, outfile)
 	int c1, c2, c3, padding;
 	FILE *outfile;
 {
@@ -322,7 +297,7 @@ base256to64 (c1, c2, c3, padding, outfile)
 	}
 }
 
-base64_encode(infile, outfile, text, length)
+void base64_encode(infile, outfile, text, length)
 	FILE *infile;
 	FILE *outfile;
 	int text, length;
@@ -353,7 +328,7 @@ static char base256[] = {
 	OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB,
     
 	OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB,
-      /*                                                -                /*/
+      /*                                                -                / */
 	OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB, OOB,OOB,OOB, 62, OOB,OOB,OOB, 63,
       /*  0   1   2   3    4   5   6   7    8   9                =        */
 	 52, 53, 54, 55,  56, 57, 58, 59,  60, 61,OOB,OOB, OOB,OOB,OOB,OOB,
@@ -368,7 +343,7 @@ static char base256[] = {
 };
 
 
-base64_decode(infile, outfile, text)
+void base64_decode(infile, outfile, text)
 	FILE *infile;
 	FILE *outfile;
 	int text;
@@ -434,7 +409,7 @@ static char From[] = "\nFrom ";
 #define DEL 127
 #define softbreak(stream) {putc(EQ, stream); putc(LF, stream);}
 
-quoted_printable_encode(infile, outfile, length)
+void quoted_printable_encode(infile, outfile, length)
 	FILE *infile;
 	FILE *outfile;
 	int length;
@@ -526,7 +501,7 @@ static char base128[] = {
 	OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB,
 };
 
-puthexchar(c1, c2, stream)
+int puthexchar(c1, c2, stream)
 	int c1, c2;
 	FILE *stream;
 {
@@ -546,7 +521,7 @@ puthexchar(c1, c2, stream)
 	}
 }
 
-quoted_printable_decode(infile, outfile)
+void quoted_printable_decode(infile, outfile)
 	FILE *infile;
 	FILE *outfile;
 {
@@ -588,7 +563,7 @@ quoted_printable_decode(infile, outfile)
 #define READ  0
 #define WRITE 1
 
-gzip64_encode(infile, outfile, text, length)
+void gzip64_encode(infile, outfile, text, length)
 	FILE *infile;
 	FILE *outfile;
 	int text, length;
@@ -633,7 +608,7 @@ gzip64_encode(infile, outfile, text, length)
 			dup(fileno(infile));
 		}
 	
-		execlp("gzip", "gzip", "-n", "-c", (char *) 0);
+		execlp("gzip", "gzip", "-q", "-n", "-c", (char *) 0);
 	
 		exit(SUCCESS);
 
@@ -659,13 +634,15 @@ gzip64_encode(infile, outfile, text, length)
 			close(READ);
 			dup(pipes2[READ]);
 
-			execlp("gzip", "gzip", "-n", "-c", (char *) 0);
+			execlp("gzip", "gzip", "-q", "-n", "-c", (char *) 0);
 	    
 			exit(SUCCESS);
 		}
 
 		/* I'm the grandchild */
-
+#ifdef OS2
+		_setmode(pipes2[WRITE],  O_BINARY);
+#endif
 		close(WRITE);
 		dup(pipes2[WRITE]);
 		close(pipes2[READ]);
@@ -682,7 +659,7 @@ gzip64_encode(infile, outfile, text, length)
     
 }
 
-gzip64_decode(infile, outfile, text)
+void gzip64_decode(infile, outfile, text)
 	FILE *infile;
 	FILE *outfile;
 	int text;
@@ -716,7 +693,7 @@ gzip64_decode(infile, outfile, text)
 				close(WRITE);
 				dup(fileno(outfile));
 			}
-			execlp ("gzip", "gzip", "-d", "-n", "-c", (char *) 0);
+			execlp ("gzip", "gzip", "-q", "-d", "-n", "-c", (char *) 0);
 		}
 	
 		exit(SUCCESS);
@@ -750,7 +727,7 @@ gzip64_decode(infile, outfile, text)
 			close(READ);
 			dup(pipes2[READ]);
 
-			execlp("gzip", "gzip", "-d", "-n", "-c", (char *) 0);
+			execlp("gzip", "gzip", "-q", "-d", "-n", "-c", (char *) 0);
 	
 			exit(SUCCESS);	    
 		}
@@ -781,7 +758,7 @@ gzip64_decode(infile, outfile, text)
 	}
 }
 
-sevenbit_signform(infile, outfile)
+void sevenbit_signform(infile, outfile)
 	FILE *infile;
 	FILE *outfile;
 {
@@ -815,19 +792,20 @@ int main (argc, argv)
 	char encode = BASE64;	/* -b -q -g */
 	int  length = LINELEN;	/* -l num */
 	int  text   = OFF;	/* -t */
-#if defined(OS2)
-	char prognametmp[256];
-	char *progname;
+	char *progname = mewbasename(argv[0]), *p;
 
-	strcpy(prognametmp, argv[0]);
-	strlwr(prognametmp);
-	progname = basename(prognametmp);
-	_fsetmode(stdin, "b");    
-	_fsetmode(stdout, "b");
-#else
-	char *progname = basename(argv[0]);
-#endif	
+	for (p = progname; *p; p++) {
+		*p = tolower(*p);
+	}
 
+#ifdef _WIN32
+	_setmode(_fileno(stdin),  O_BINARY);
+	_setmode(_fileno(stdout), O_BINARY);	
+#endif
+#ifdef OS2
+	_setmode(fileno(stdin),  O_BINARY);
+	_setmode(fileno(stdout), O_BINARY);	
+#endif
 	while((optc = getopt_long(argc, argv, "esdbqgzl:thv", longopts,
 				  (int *)0)) != EOF) {
 		switch (optc) {
@@ -952,3 +930,37 @@ int main (argc, argv)
 
 	exit(SUCCESS);
 }
+
+/* 
+ * Copyright (C) 1994, 1995, 1996, 1997 Mew developing team.
+ * All rights reserved.
+ * 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of the team nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+ * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE TEAM OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
+ * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+/*
+ * mewencode.c ends here
+ */
