@@ -25,13 +25,29 @@
 ;;
 
 (eval-when-compile 
-  (require 'gnus))
+  (require 'gnus)
+  (if (not (or (string-match "^GNUS [34]" gnus-version)
+	       (string-match "^Gnus v5.0" gnus-version)
+	       (string-match "^5.[0-3]" gnus-version-number)))
+      (require 'gnus-sum)))
 
 (require 'mew)
 
-(defvar mh-lib "/usr/local/lib/mh")
+(defvar mew-prog-imstore "imstore")
+(defvar mew-prog-imstore-arg "--dst=%s")
+;(defvar mew-prog-imstore "/usr/local/lib/mh/rcvstore")
+;(defvar mew-prog-imstore-arg "%s")
 
-(defvar mew-gnus-save-fixed-folder nil)
+(defvar mew-gnus-save-fixed-folder nil
+  "*If specified, always use it as a candidate to save article.")
+
+(defvar mew-gnus-save-preserve-dot t
+  "*If nil, use hierarchical directory to save article.")
+
+(defun mew-gnus-newsgroup-name ()
+  (if mew-gnus-save-preserve-dot
+      gnus-newsgroup-name
+    (gnus-newsgroup-directory-form gnus-newsgroup-name)))
 
 (defun gnus-summary-save-in-mew (&optional folder)
   "Save this article to MH folder (using `rcvstore' in MH library).
@@ -43,37 +59,40 @@ Optional argument FOLDER specifies folder name."
     (save-restriction
       (widen)
       (or mew-folder-alist
-	  (setq mew-folder-list (mew-folder-make-list)
+	  (setq mew-folder-list (mew-folder-make-list nil)
 		mew-folder-alist (mew-folder-make-alist mew-folder-list)))
       (let ((folder
 	     (or folder
 		 (mew-input-folder
 		  (or mew-gnus-save-fixed-folder
-		      (car (mew-guess-by-alist))
-		      (concat "+" gnus-newsgroup-name)))))
+		      (car (mew-refile-guess-by-alist))
+		      (concat "+" (mew-gnus-newsgroup-name))))))
 	    (errbuf (get-buffer-create " *GNUS rcvstore*")))
 	(if (not (equal (aref folder 0) ?+))
 	    (message (format 
 		      "First letter of '%s' must be '+'."
 		      folder))
-	  (unwind-protect
-	      (call-process-region (point-min) (point-max)
-				   (expand-file-name "rcvstore" mh-lib)
-				   nil errbuf nil folder)
-	    (set-buffer errbuf)
-	    (if (zerop (buffer-size))
-		(message "Article saved in folder: %s" folder)
-	      (message "%s" (buffer-string)))
-	    (kill-buffer errbuf)))))))
+	  (if (mew-folder-check folder)
+	      (unwind-protect
+		  (mew-piolet
+		   mew-cs-infile mew-cs-outfile
+		   (call-process-region (point-min) (point-max)
+					mew-prog-imstore nil errbuf nil
+					(format mew-prog-imstore-arg folder)))
+		(set-buffer errbuf)
+		(if (zerop (buffer-size))
+		    (message "Article saved in folder: %s" folder)
+		  (message "%s" (buffer-string)))
+		(kill-buffer errbuf))))))))
 
 (defun mew-gnus-post-news ()
   "Post a news using mew."
   (interactive)
-  (or (bufferp mew-buffer-tmp)
+  (or mew-temp-dir
       (save-excursion
 	(mew-set-environment)))
-  (let ((file (mew-draft-get-new)))
-    (if (null mew-path) (mew-init))
+  (let ((file (mew-folder-new-message mew-draft-folder)))
+    (if (null mew-mail-path) (mew-init))
     (mew-current-set 'window (current-window-configuration))
     (delete-other-windows)
     (switch-to-buffer (find-file-noselect file))
@@ -87,23 +106,24 @@ Optional argument FOLDER specifies folder name."
   "Reply or followup to GNUS article using mew.
 Optional argument YANK means yank original article."
   (interactive)
-  (or (bufferp mew-buffer-tmp)
+  (or mew-temp-dir
       (save-excursion
 	(mew-set-environment)))
-  (let ((file (mew-draft-get-new))
+  (let ((file (mew-folder-new-message mew-draft-folder))
 	from cc subject date to reply-to newsgroups in-reply-to references
 	distribution)
-    (if (null mew-path) (mew-init))
+    (if (null mew-mail-path) (mew-init))
     (mew-current-set 'window (current-window-configuration))
     (delete-other-windows)
     (gnus-summary-display-article (gnus-summary-article-number) t) ;; redisplay
     (pop-to-buffer gnus-article-buffer)
-    (search-forward "\n\n" nil t)
-    (push-mark (point) t t)
     (goto-char (point-max))
+    (push-mark (point) t t)
+    (goto-char (point-min))
+    (search-forward "\n\n" nil t)
     (let ((split-window-keep-point t))
       (split-window-vertically))
-    (setq from (gnus-fetch-field "From")
+    (setq from (mew-addrstr-parse-address-list (gnus-fetch-field "From"))
 	  subject (let ((subject (gnus-fetch-field "Subject")))
 		    (if (and subject
 			     (not (string-match "^[Rr][Ee]:.+$" subject)))
@@ -129,7 +149,10 @@ Optional argument YANK means yank original article."
 	  (insert (concat "Distribution: " distribution "\n"))))
     (make-variable-buffer-local 'mew-message-citation-buffer) 
     (setq mew-message-citation-buffer gnus-article-buffer))
-  (if yank (mew-draft-cite)))
+  (if yank
+      (progn
+	(goto-char (point-max))
+	(mew-draft-cite))))
 
 (defun mew-gnus-reply-with-citation ()
   "Reply or followup to GNUS article using mew.
@@ -140,7 +163,7 @@ Original article is yanked automatically."
 (defun mew-gnus-mail-forward (&optional buffer)
   "Forward the current message to another user using mew."
   (interactive)
-  (or (bufferp mew-buffer-tmp)
+  (or mew-temp-dir
       (save-excursion
 	(mew-set-environment)))
   (mew-current-set 'window (current-window-configuration))
@@ -149,20 +172,21 @@ Original article is yanked automatically."
 		     gnus-article-buffer))
   (let* ((subject (concat "[" gnus-newsgroup-name "] "
 			  (or (gnus-fetch-field "subject") "")))
-	 (file (mew-draft-get-new))
+	 (file (mew-folder-new-message mew-draft-folder))
 	 (mimefolder (mew-draft-to-mime file))
-         (mimedir (mew-expand-file-name mimefolder)))
+         (mimedir (mew-expand-folder mimefolder)))
     (if (null (file-directory-p mimedir))
         (mew-make-directory mimedir)
       (if (null (mew-directory-empty-p mimedir))
-          (if (mew-y-or-n-p (format "Mime folder %s is not empty. Delete it? "
-				    mimefolder))
+          (if (y-or-n-p (format "Mime folder %s is not empty. Delete it? "
+				mimefolder))
               (progn
                 (call-process "rm" nil nil nil "-rf" mimedir)
                 (mew-make-directory mimedir)))))
     (unwind-protect
 	(progn
-	  (write-region (point-min) (point-max) (mew-draft-get-new mimefolder))
+	  (write-region (point-min) (point-max)
+			(mew-folder-new-message mimefolder))
 	  (let ((split-window-keep-point t))
 	    (split-window-vertically))
 	  (switch-to-buffer-other-window (find-file-noselect file))
