@@ -1,89 +1,15 @@
-;;;			    mew-decode.el
-;;;
-;;;	      Copyright (C) 1996-1997  Kazuhiko Yamamoto
-;;;
-;;;		   This emacs lisp library conforms
-;;;		GNU GENERAL PUBLIC LICENSE Version 2.
-;;;
-;;; Author:  Kazuhiko Yamamoto <kazu@is.aist-nara.ac.jp>
-;;; Created: October  2, 1996
-;;; Revised: April   21, 1997
-;;;
+;;; mew-decode.el --- MIME syntax decoder for Mew
 
-(defconst mew-decode-version "mew-decode.el version 0.12")
+;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Created: Oct  2, 1996
+;; Revised: Aug 24, 1998
+
+;;; Code:
+
+(defconst mew-decode-version "mew-decode.el version 0.24")
 
 (require 'mew)
 
-(defvar mew-file-max-size 100000
-  "*The max size of messages. If the size of a message is greater
-than mew-file-max-size, Mew asks the user to insert the entire file
-or truncate it.")
-
-(defvar mew-header-max-length 50
-  "*If the length of a header exceeds this value, 
-it is not arranged nor MIME decoded.
-See also mew-header-max-depth.")
-
-(defvar mew-field-visible
-  '("Subject:" "From:" "To:" "Apparently-To:"
-    "Cc:" "Newsgroups:" "Date:" "Reply-To:"
-    "Resent-From:" "Resent-To:" "Resent-Cc:"
-    "Mime-Version:" "Content-Type:" "Content-Transfer-Encoding:")
-  "*Visible fields when a message is displayed in a message mode.")
-
-(defvar mew-field-invisible
-  '("Received:" "Return-Path:" "Sender:" "Errors-To:"
-    "Resent-Date:" "Message-Id:" "Resent-Message-Id:"
-    "Resent-Sender:" "Resent-Reply-To:" "Delivery-Date:")
-  "*These fields are hidden when a message is displayed over a message
-buffer. Back-scroll makes them visible.")
-
-(defvar mew-field-other-visible t
-  "*If *non-nil*, other fields than fields of mew-field-visible and
-that of mew-field-invisible are displayed after that of mew-field-visible.
-Otherwise, they are hidden over a message buffer. That is, 
-fields of mew-field-visible are visible only.")
-
-(defvar mew-rfc822-fields
-   '(("To:" . nil)
-     ("Cc:" . nil)
-     ("From:" . t)
-     ("Resent-To:" . nil)
-     ("Resent-Cc:" . nil)
-     ("Resent-From:" . t)
-     ("Reply-To:" . nil)
-     ("Subject:" . t))
-   "Key and del pair alist for decoding. 
-Key is one of RFC822 fields not of MIME fields.
-If del is t, linear-white-spaces for folding are removed.
-See also mew-mime-fields."
-   )
-
-(defvar mew-field-visible-regex nil)
-(defvar mew-field-invisible-regex nil)
-(defvar mew-field-visible-length 0)
-(defvar mew-keyval "^\\([^ \t:]+:\\)[ \t]*\\(.*\\)$")
-
-(defconst mew-ct-txt "Text/Plain")
-(defconst mew-ct-msg "Message/Rfc822")
-(defconst mew-ct-ext "Message/External-Body")
-(defconst mew-ct-mlm "Multipart/Mixed")
-(defconst mew-ct-mls "Multipart/Signed")
-(defconst mew-ct-mle "Multipart/Encrypted")
-(defconst mew-ct-mld "Multipart/Digest")
-(defconst mew-ct-ado "Audio/Basic")
-(defconst mew-ct-apo "Application/Octet-Stream")
-(defconst mew-ct-pgs "application/pgp-signature") ;; xxx case
-(defconst mew-ct-pge "application/pgp-encrypted") ;; xxx case
-(defconst mew-ct-apk "Application/Pgp-keys")
-
-(defconst mew-type-txt (list mew-ct-txt "charset=us-ascii"))
-(defconst mew-type-msg (list mew-ct-msg))
-(defconst mew-type-mlm (list mew-ct-mlm))
-(defconst mew-type-ado (list mew-ct-ado))
-(defconst mew-type-apo (list mew-ct-apo))
-(defconst mew-type-apk (list mew-ct-apk))
-
 (defvar mew-prog-mime-decode-switch
   (list (cons mew-b64 '("-b"))
 	(cons mew-qp  '("-q"))
@@ -96,12 +22,15 @@ See also mew-mime-fields."
 
 (defvar mew-decode-multipart-encrypted-switch
   '(("application/pgp-encrypted"  . mew-pgp-decrypt)
-    ("application/moss-keys"      . mew-moss-decrypt))
+;;    ("application/moss-keys"      . mew-moss-decrypt)
+    )
   )
 
 (defvar mew-decode-multipart-signed-switch
-  '(("application/pgp-signature"  . mew-pgp-verify)
-    ("application/moss-signature" . mew-moss-verify))
+  '(("application/pgp-signature"   . mew-pgp-verify)
+;;    ("application/moss-signature"  . mew-moss-verify)
+;;    ("application/pkcs7-signature" . mew-smime-verify)
+    )
   )
 
 (defvar mew-decode-multipart-protocol-prog
@@ -116,7 +45,7 @@ See also mew-mime-fields."
 ;;
 
 (defmacro mew-decode-error (error-msg)
-  (` (progn (setq mew-decode-error (, error-msg)) (error))))
+  (` (progn (setq mew-decode-error (, error-msg)) (error ""))))
 
 (defun mew-decode-rfc822-header ()
   ;; Called on the beginning of the header in the narrowed region
@@ -124,180 +53,170 @@ See also mew-mime-fields."
   ;; (2) Delete X-Mew: fields
   ;; (3) Arrange decoded-RFC822-fields, mew-mv:, MIME fields in order
   ;; Return whether MIME or not
-  (if (not (re-search-forward mew-eoh nil t))
-      (mew-decode-error "No end-of-header for RFC822 header"))
+  (if (re-search-forward mew-eoh nil t)
+      (beginning-of-line)
+    (mew-decode-error "No end-of-header for RFC822 header"))
   (if (> (count-lines (point-min) (point)) mew-header-max-length)
-      (error "Too long header")
-    (let ((case-fold-search t)
-	  key beg val value attr cnt-hdrs mv back)
-      (save-restriction
-	(narrow-to-region (point-min) (point))
-	(goto-char (point-min))
-	(while (not (eobp))
-	  (if (not (looking-at mew-keyval))
-	      (mew-decode-error "'key: value' pair not found"))
-	  (setq key (mew-match 1))
-	  (setq val (match-beginning 2))
+      (mew-decode-error "Too long header"))
+  (let (key beg med cnt-hdrs back subj mimep type structp)
+    (save-restriction
+      (narrow-to-region (point-min) (point))
+      (goto-char (point-min))
+      (while (not (eobp))
+	(if (not (looking-at mew-keyval))
+	    (forward-line)
+	  (setq key (capitalize (mew-match 1)))
 	  (setq beg (match-beginning 0))
+	  (setq med (match-end 0))
 	  (forward-line)
-	  (while (looking-at mew-lwsp) (forward-line))
+	  (mew-header-goto-next)
 	  (cond
-	   ((setq attr (mew-assoc key mew-rfc822-fields 0 t))
-	    (forward-char -1)
-	    (setq value (mew-buffer-substring val (point)))
-	    (delete-region val (point))
-	    (insert (mew-header-decode value (cdr attr)))
-	    (forward-line))
-	   ((string-match key mew-mv:)
-	    ;; MIME-Version:
-	    (setq mv (buffer-substring beg (point)))
-	    (delete-region beg (point)))
-	   ((string-match key mew-x-mew:)
+	   ((mew-case-equal key mew-x-mew:)
 	    ;; deleting X-Mew: on the RFC822 header
 	    (delete-region beg (point)))
-	   ((mew-assoc key mew-mime-fields 0 t)
+	   ((string-match "^Content-" key)
 	    ;; save MIME fields
 	    (setq cnt-hdrs (concat cnt-hdrs (buffer-substring beg (point))))
 	    (delete-region beg (point)))
-	   )
-	  ))
-      (if mv (insert mv))
-      (if (not cnt-hdrs)
-	  ()
-	(setq back (point))
-	(insert cnt-hdrs)
-	(goto-char back))
-      ;; the beginning of the content header
-      (if mv (string-match (concat mew-mv: "[ \t]*" mew-mime-version) mv))
-      )
-    ))
-
-(defun mew-position (list a)
-  (let ((n 0)
-	(ret nil))
-    (catch 'loop 
-      (while list 
-	(if (equal (downcase (car (car list))) (downcase a))
-	    (throw 'loop (setq ret n)))
-	(setq n (1+ n))
-	(setq list (cdr list))
-	)
-      )
-    ret
-    ))
+	   (t
+	    (setq type (mew-field-type key))
+	    (setq structp (or (equal type 'mailbox) (equal type 'mime)))
+	    (mew-header-decode-region structp med (point))
+	    (cond
+	     ((mew-case-equal key mew-subj:)
+	      (setq subj (buffer-substring med (1- (point)))))
+	     ((mew-case-equal key mew-mv:)
+	      ;; MIME-Version:
+	      (setq mimep (string-match
+			   mew-mv:-num
+			   (mew-addrstr-parse-value
+			    (buffer-substring med (point))))))))))
+	))
+    (if (not cnt-hdrs)
+	()
+      (setq back (point))
+      (insert cnt-hdrs)
+      (goto-char back))
+    ;; the beginning of the content header
+    ;; return value
+    (list mimep subj)))
 
 (defun mew-decode-mime-header (&optional dct)
   ;; Called on the beginning of the content header in the narrowed region
   ;; Return a part syntax after moving the beginning of the content body
-  (if (not (re-search-forward mew-eoh))
-      (mew-decode-error "No end-of-header for MIME header"))
+  (if (re-search-forward mew-eoh)
+      (beginning-of-line)
+    (mew-decode-error "No end-of-header for MIME header"))
   (if (> (count-lines (point-min) (point)) mew-header-max-length)
-      (error "Too long header")
-    (let* ((case-fold-search t)
-	   (len (length mew-mime-fields))
-	   (vec (make-vector len nil))
-	   key val position attr value)
-      (aset vec 0 (if dct dct mew-type-txt))
-      (save-restriction
-	(narrow-to-region (point-min) (point))
-	(goto-char (point-min))
-	(while (not (eobp))
-	  (if (not (looking-at mew-keyval))
-	      (mew-decode-error "'key: value' pair not found"))
-	  (setq key (mew-match 1))
-	  (setq val (match-beginning 2))
+      (mew-decode-error "Too long header"))
+  (let* ((case-fold-search t)
+	 (len (length mew-mime-fields))
+	 (vec (make-vector len nil))
+	 key med attr n act value)
+    (aset vec 0 (if dct dct mew-type-txt))
+    (save-restriction
+      (narrow-to-region (point-min) (point))
+      (goto-char (point-min))
+      (while (not (eobp))
+	(if (not (looking-at mew-keyval))
+	    (forward-line)
+	  (setq key (capitalize (mew-match 1)))
+	  (setq med (match-end 0))
 	  (forward-line)
-	  (while (looking-at mew-lwsp) (forward-line))
-	  (if (not (setq position (mew-position mew-mime-fields key)))
+	  (mew-header-goto-next)
+	  (setq attr (assoc key mew-mime-fields))
+	  (if (not attr)
 	      ()
-	    (setq attr (cdr (mew-assoc key mew-mime-fields 0 t)))
-	    (setq value (mew-buffer-substring val (point)))
+	    (setq n (nth 1 attr))
+	    (setq act (nth 2 attr))
+	    (setq value (buffer-substring med (1- (point))))
 	    (cond
-	     ((equal attr 'analyze)
-	      (setq value (mew-header-syntax-list value))) ;; list
-	     ((equal attr 'extract)
-	      (setq value (mew-header-syntax value)))
-	     ((equal attr 'decode)
-	      (setq value (mew-header-decode value nil))
-	      (delete-region val (point))
-	      (insert value)
-	      (setq value (substring value 0 -1)) ;; delete \n
-	      )
+	     ((equal act 'analyze)
+	      (setq value (mew-addrstr-parse-value-list value)))
+	     ((equal act 'extract)
+	      (setq value (mew-addrstr-parse-value value)))
+	     ((equal act 'decode)
+	      (mew-header-decode-region nil med (point) t)
+	      ;; mew-header-decode-region goes to the max point in
+	      ;; the narrowed region. So, this must be (point).
+	      (setq value (buffer-substring med (1- (point)))))
 	     )
-	    (aset vec position value)
-	    )
-	  ))
-      (forward-line) 
-      ;; the beginning of the content body
-      (vconcat (list 'single (point) nil nil) vec)
-      )
-    ))
+	    (aset vec n value)))))
+    (forward-line) 
+    ;; the beginning of the content body
+    (vconcat (list 'single (point) nil nil) vec)))
 
 (defun mew-decode-mime (enc &optional text charset)
   ;; Called on the beginning of the content body in the narrow region
   ;; Decode the content body
   ;; Convert text
-  (let ((beg (point))
-	(switch mew-prog-mime-decode-switch)
-	opt file)
+  (let* ((beg (point))
+	 (switch mew-prog-mime-decode-switch)
+	 (cs (mew-charset-to-cs charset))
+	 (post-conv (if mew-temacs-p (mew-cs-post-conv cs)))
+	 opt file)
     (if (null enc)
 	() ;; just in case
       (if text (setq switch mew-prog-mime-decode-text-switch))
-      (setq opt (cdr (mew-assoc-match enc switch 0)))
+      (setq opt (cdr (mew-assoc-case-equal enc switch 0)))
       ;; opt is nil if 7bit, 8bit, and binary
       (if opt
-	  (mew-flet 
-	   ;; NEVER use call-process-region for privary reasons
-	   (if (not (file-exists-p mew-temp-dir))
-	       (mew-make-directory mew-temp-dir)) ;; just in case
-	   (setq file (make-temp-name mew-temp-file))
-	   (write-region beg (point-max) file)
-	   (message "") ;; flush the message
-	   (delete-region beg (point-max))
-	   ;; input is always in ASCII domain
-	   (mew-plet
-	    (apply (function call-process) mew-prog-mime-decode 
-		   file t nil opt)
-	    )
-	   (if (file-exists-p file) (delete-file file))
-	   ))
-      )
-    (mew-cs-decode-region beg (point-max) charset 'charset)
-    ))
+	  (if (not (mew-which mew-prog-mime-decode exec-path))
+	      (mew-decode-error (concat mew-prog-mime-decode " doesn't exist"))
+	    (setq file (mew-make-temp-name))
+	    (mew-flet 
+	     ;; NEVER use call-process-region for privacy reasons
+	     (write-region beg (point-max) file nil 'no-msg))
+	    (delete-region beg (point-max))
+	    ;; input is always in ASCII domain
+	    (mew-plet
+	     (apply (function call-process) mew-prog-mime-decode 
+		    file t nil opt))
+	    (if (file-exists-p file) (delete-file file)))))
+    (mew-cs-decode-region beg (point-max) cs)
+    (if post-conv
+	(let ((mc-flag t))
+	  (funcall post-conv beg (point-max))))))
 
 ;;
 ;; Kick start function
 ;;
 
-(defun mew-file-size (file)
-  (nth 7 (file-attributes file))
-  )
+(defmacro mew-file-size (file)
+  (` (nth 7 (file-attributes (, file)))))
 
-(defun mew-decode (file &optional edit)
+(defun mew-decode (fld msg &optional edit)
   ;; in cache buffer
-  (setq mew-decode-error nil)
-  ;; must be set mc-flag on Mule 2 for re-search-forward
-  (if (and mew-emacs-p mew-mule-p) (setq mc-flag nil))
-  (widen)
-  (erase-buffer)
-  (mew-flet
-   (let ((size (mew-file-size file)))
-     (if (and (> size mew-file-max-size)
-	      (mew-y-or-n-p
-	       (format "This mail size is %s. Truncate it? " size)))
-	 (setq size mew-file-max-size))
-     (insert-file-contents file nil 0 size)
-     ))
+  (mew-erase-buffer)
   (goto-char (point-min))
+  (setq mew-decode-error nil)
+  (cond
+   ((or (mew-folder-newsp fld) (mew-folder-imapp fld))
+    (mew-piolet mew-cs-noconv-eol mew-cs-noconv
+     (mew-im-call-process nil mew-prog-imcat (format "--src=%s" fld) msg)))
+   (t
+    (let ((file (mew-expand-folder fld msg)))
+      (mew-frwlet mew-cs-noconv-eol mew-cs-noconv
+	 (insert-file-contents file))
+      (setq mew-cache-attribute (mew-cache-attribute-get file))))
+   )
+  ;; afer reading the file
+  (if mew-mule-p
+      (cond
+       ((fboundp (function set-buffer-multibyte))
+	(set-buffer-multibyte t))
+       (mew-temacs-p
+	(setq mc-flag nil) ;; for re-search-forward
+	)))
   ;; Illegal messages may not have end-of-header.
   ;; Truncated messages may not have end-of-header.
   (if (re-search-forward mew-eoh nil t)
-      () ;; OK
+      ()
     (setq mew-decode-error "No end-of-header(null line) in the top level")
     (goto-char (point-max))
-    (insert "\n\n"))
+    (if (not (bolp)) (insert "\n"))
+    (insert "\n"))
   (goto-char (point-min))
-  ;; the beginning of the message
   (if mew-debug
       (let ((debug-on-error t))
 	(setq mew-decode-syntax
@@ -314,20 +233,13 @@ See also mew-mime-fields."
 	       (mew-decode-syntax-rfc822-head) edit))
       (error
        (widen)
-       (goto-char (point-min))
-       ;; There is certainly end-of-header but just in case.
-       (re-search-forward mew-eoh nil t)
-       (forward-line)
-       ;; the beginning of the body
+       (mew-header-goto-body)
        (setq mew-decode-syntax (mew-decode-syntax-rfc822))
-       ;; min, point-1, point, point-max
-       )
-      ))
-  (setq mew-cache-attribute (mew-cache-attribute-get file))
-  )
+       ;; min, point - 1, point, point-max
+       ))))
 
 ;;
-;; function "m"
+;; the function "m":: for message
 ;;
 
 (defun mew-decode-message (syntax &optional edit)
@@ -346,9 +258,11 @@ See also mew-mime-fields."
   ;;(end)
   ;;     Content-Body                    == physical content body
   ;;(max)
-  (let ((mimep (mew-decode-rfc822-header))
-	;; the beginning of the physical content header (cur)
-	part)
+  (let* ((m-s (mew-decode-rfc822-header))
+	 (mimep (nth 0 m-s))
+	 (subj (nth 1 m-s))
+	 ;; the beginning of the physical content header (cur)
+	 part)
     (cond 
      (mimep ;; MIME
       (save-restriction
@@ -358,25 +272,25 @@ See also mew-mime-fields."
 	;; the physical content body
 	(mew-syntax-set-key syntax 'message)
 	(mew-syntax-set-end syntax (1- (mew-syntax-get-begin part)))
-	(mew-syntax-cat syntax part) ;; the return value
+	(or (mew-syntax-get-cd syntax) (mew-syntax-set-cd syntax subj))
+	(mew-syntax-cat syntax part) ;; return value
 	))
      (t ;; RFC822
       ;; the beginning of the meaningless physical content header
-      (if (not (re-search-forward mew-eoh nil t))
+      (if (re-search-forward mew-eoh nil t)
+	  (forward-line)
 	(mew-decode-error "No end-of-header(null line) in RFC822 message"))
-      (forward-line)
       ;; the beginning of the BODY(i.e. the physical content body)
       (mew-cs-decode-region (point) (point-max) mew-cs-rfc822-trans)
       (mew-syntax-set-key syntax 'message)
       (mew-syntax-set-end syntax (1- (point)))
+      (or (mew-syntax-get-cd syntax) (mew-syntax-set-cd syntax subj))
       (mew-decode-syntax-rfc822 syntax)
       ;; (point-min), (point) - 1, (point), (point-max)
-      )
-     )
-    ))
+      ))))
 
 ;;
-;; function "S"
+;; the function "S":: for singlepart
 ;;
 
 (defun mew-decode-singlepart (&optional dct parent edit)
@@ -386,89 +300,83 @@ See also mew-mime-fields."
 	 (syntax (mew-decode-mime-header dct))
 	 ;; the beginning of the content body
 	 (ctl (mew-syntax-get-ct syntax))
-	 (ct (car ctl))
-	 (cte (mew-syntax-get-cte syntax))
+	 (ct (capitalize (car ctl)))
+	 (cte (or (mew-syntax-get-cte syntax) mew-7bit))
+	 (encap nil)
 	 charset)
-    (cond 
+    (cond
+     ((not (mew-member-case-equal cte mew-decode-value))
+      (mew-syntax-set-ct syntax mew-type-apo))
      ;; Message/Rfc822, decoding is not required
-     ((string-match mew-ct-msg ct)
-      (if (null edit)
-	  (save-restriction
-	    (narrow-to-region (point) (point-max))
-	    (if (equal parent 'message)
-		;; Rfc822 directly under rfc822 since both his parent 
-		;; and his child are rfc822. It must be multipart on Mew!
-		;; Otherwise, Mew cannot display the syntax.
-		(let ((head mew-encode-syntax-multi-head)
-		      (part (mew-decode-message syntax)))
-		  ;; begin for multipart syntax is important because
-		  ;; the begin will be used by the parent to set hend
-		  (mew-syntax-set-begin head (mew-syntax-get-begin part))
-		  (mew-syntax-set-end head (point-max))
-		  (mew-syntax-cat head part))
-	      (mew-decode-message syntax)))
-	;; return without analyzing this message
-	(mew-syntax-set-end syntax (point-max))
-	syntax ;; the return value
-	))
-     ((string-match mew-ct-ext ct)
-      (let* ((access-type (mew-syntax-get-member ctl "access-type"))
-	     (func (cdr (mew-assoc access-type mew-ext-include-switch 0 t))))
+     ((mew-case-equal mew-ct-msg ct)
+      (if edit
+	  ()
+	(if (equal parent 'message) (setq encap t))
+	(save-restriction
+	  (narrow-to-region (point) (point-max))
+	  (setq syntax (mew-decode-message syntax)))))
+     ((mew-case-equal mew-ct-ext ct)
+      (let* ((at (mew-syntax-get-member ctl "access-type"))
+	     (func (cdr (mew-assoc-case-equal at mew-ext-include-switch 0))))
 	(if (not (and func (fboundp func)))
-	    (progn
-	      ;; probably cte is 7bit
-	      (mew-syntax-set-end syntax (point-max))
-	      syntax ;; the return value
-	      )
+	    ()
 	  (save-excursion
 	    (goto-char (point-max)) ;; phantom body
-	    (funcall func ctl)
-	    )
+	    (funcall func ctl))
 	  (delete-region begin (point))
 	  ;; xxx again how about edit?
-	  (mew-decode-singlepart)))
-      )
+	  (setq syntax (mew-decode-singlepart)))))
      ;; Multipart, decoding is not required
-     ((string-match "^Multipart" ct)
-      (let* ((protocol (mew-syntax-get-member ctl "protocol"))
+     ((string-match "^Multipart/" ct)
+      (let* ((proto (mew-syntax-get-member ctl "protocol"))
 	     (switch mew-decode-multipart-protocol-prog) ;; save length
-	     (prog (if protocol (cdr (mew-assoc protocol switch 0 t))))
+	     (prog (if proto (cdr (mew-assoc-case-equal proto switch 0))))
 	     (existp (if prog (mew-which prog exec-path))))
 	(cond 
-	 ((string-match mew-ct-mld ct)
+	 ((mew-case-equal mew-ct-mld ct)
 	  ;; semantics into digest
-	  (mew-decode-multipart syntax mew-type-msg edit))
-	 ((and (string-match mew-ct-mls ct) existp)
-	  (mew-decode-multipart-signed syntax edit))
-	 ((and (string-match mew-ct-mle ct) existp)
-	  (mew-decode-multipart-encrypted syntax edit))
+	  (setq syntax (mew-decode-multipart syntax mew-type-msg edit)))
+	 ((and (mew-case-equal mew-ct-mls ct) existp)
+	  (setq syntax (mew-decode-multipart-signed syntax edit)))
+	 ((and (mew-case-equal mew-ct-mle ct) existp)
+	  (setq syntax (mew-decode-multipart-encrypted syntax edit)))
 	 (t
-	  (mew-decode-multipart syntax nil edit))
-	 ))
-      )
+	  (setq syntax (mew-decode-multipart syntax nil edit)))
+	 )))
      ;; Others
-     (t  
-      (if (null cte) (setq cte "7bit"))
+     (t
+      (if (and (equal parent 'message) (not (mew-case-equal mew-ct-txt ct)))
+	  (setq encap t))
       ;; even if cte is nil, call mew-decode-mime for charset conversion
       (cond 
        ((string-match "^Text/" ct)
 	(setq charset (mew-syntax-get-member ctl "charset"))
 	;;charset is "us-ascii" if it is nil.
 	(mew-decode-mime cte 'text charset))
-       ((mew-member-match ct mew-mime-content-type-text-list)
+       ((mew-member-case-equal ct mew-mime-content-type-text-list)
 	(mew-decode-mime cte 'text))
        (t
-	(mew-decode-mime cte)
-	)
+	(mew-decode-mime cte))
        )
-      (mew-syntax-set-end syntax (point-max))
-      syntax ;; the return value
       )
      )
+    (if (or (not (mew-case-equal mew-ct-msg ct)) edit)
+	(mew-syntax-set-end syntax (point-max)))
+    (if encap
+	;; Mew allows text/plain and multipart/* for body.
+	;; If other CT: is embedded under message, it should be
+	;; encapsulated in multipart/mixed.
+	(let ((head mew-encode-syntax-multi-head))
+	  ;; begin for multipart syntax is important because
+	  ;; the begin will be used by the parent to set hend
+	  (mew-syntax-set-begin head (mew-syntax-get-begin syntax))
+	  (mew-syntax-set-end head (point-max))
+	  (mew-syntax-cat head syntax)) ;; return value
+      syntax) ;; return value
     ))
 
 ;;
-;; function "M"
+;; the function "M":: for multipart
 ;;
 
 (defun mew-decode-multipart (syntax &optional dct edit)
@@ -478,7 +386,7 @@ See also mew-mime-fields."
 	 (parts []) part
 	 obound ebound bregex start break)
     (if (null boundary)
-	(error "No boundary parameter for multipart"))
+	(mew-decode-error "No boundary parameter for multipart"))
     (mew-syntax-set-key syntax 'multi)
     (setq obound (concat "--" boundary))
     (setq ebound (concat "--" boundary "--"))
@@ -507,15 +415,10 @@ See also mew-mime-fields."
 	    ))
 	(setq start (point)) ;; the beginning of the part
 	(if break 
-	    (progn
-	      (mew-syntax-set-end syntax (point-max)) ;; not necessary
-	      (throw 'multipart (vconcat syntax parts)))
-	  )
-	))
-    ))
+	    (throw 'multipart (vconcat syntax parts)))))))
 
 ;;
-;; function "P"
+;; the function "D":: for decryption
 ;;
 
 (defun mew-decode-multipart-encrypted (syntax &optional edit)
@@ -533,14 +436,14 @@ See also mew-mime-fields."
 	 (boundary (regexp-quote (mew-syntax-get-member ctl "boundary")))
 	 (protocol (mew-syntax-get-member ctl "protocol"))
 	 (switch mew-decode-multipart-encrypted-switch) ;; save length
-	 (func (cdr (mew-assoc protocol switch 0 t)))
+	 (func (cdr (mew-assoc-case-equal protocol switch 0)))
 	 file1 file2 file3 syntax3
 	 start result file3result privacy
 	 oregex eregex)
     (if (null boundary)
-	(error "No boundary parameter for multipart"))
+	(mew-decode-error "No boundary parameter for multipart"))
     (if (null protocol)
-	(error "No protocol parameter for security multipart"))
+	(mew-decode-error "No protocol parameter for security multipart"))
     (setq oregex (concat "^--" boundary "$"))
     (setq eregex (concat "^--" boundary "--$"))
     ;;
@@ -580,9 +483,9 @@ See also mew-mime-fields."
     (goto-char (point-min))
     (setq syntax3 (mew-decode-singlepart nil nil edit))
     (setq privacy (mew-syntax-get-privacy syntax3))
-    (if (and privacy (not (equal privacy "")))
-	(setq result (concat result "\n\t" privacy)))
-    (mew-syntax-set-privacy syntax3 result)
+    (if privacy (setq result (concat result "\n\t")))
+    (mew-syntax-set-privacy syntax3
+			    (cons (list mew-ct-mle protocol result) privacy))
     ;; Throw away garbage
     (if (file-exists-p file1) (delete-file file1))
     (if (file-exists-p file2) (delete-file file2))
@@ -590,6 +493,10 @@ See also mew-mime-fields."
     syntax3 ;; return value
     ))
 
+;;
+;; the function "V":: for verification
+;;
+
 (defun mew-decode-multipart-signed (syntax &optional edit)
   ;; called in narrowed region
   ;;
@@ -604,14 +511,15 @@ See also mew-mime-fields."
 	 (ctl (mew-syntax-get-ct syntax))
 	 (boundary (regexp-quote (mew-syntax-get-member ctl "boundary")))
 	 (protocol (mew-syntax-get-member ctl "protocol"))
+	 (micalg (mew-syntax-get-member ctl "micalg"))
 	 (switch mew-decode-multipart-signed-switch) ;; save length
-	 (func (cdr (mew-assoc protocol switch 0 t)))
+	 (func (cdr (mew-assoc-case-equal protocol switch 0)))
 	 file1 file2 syntax3 end1 start2 result privacy
 	 oregex eregex)
     (if (null boundary)
-	(error "No boundary parameter for multipart"))
+	(mew-decode-error "No boundary parameter for multipart"))
     (if (null protocol)
-	(error "No protocol parameter for security multipart"))
+	(mew-decode-error "No protocol parameter for security multipart"))
     (setq oregex (concat "^--" boundary "$"))
     (setq eregex (concat "^--" boundary "--$"))
     ;;
@@ -629,8 +537,12 @@ See also mew-mime-fields."
     (forward-line) ;; the beginning of the key part
     (setq start2 (point)) 
     ;;
-    (setq file1 (mew-save-signature-form (point-min) end1))
-    ;;
+    (save-excursion
+      (save-restriction
+	(narrow-to-region (point-min) end1)
+	(setq file1 (mew-save-transfer-form (point-min) end1 'retain))))
+    ;; xxx make use of micalg?
+    ;; xxx protocol check?
     (if (not (re-search-forward eregex))
 	(mew-decode-error "No third boundary for Multipart/Signed"))
     (beginning-of-line) ;; the end of the encrypted part + 1
@@ -639,14 +551,14 @@ See also mew-mime-fields."
     (delete-region end1 (point-max))
     ;; Now the signed part only
     ;; Call protocl function
-    (setq result (funcall func file1 file2))
+    (setq result (funcall func file1 file2 micalg))
     ;; Analyze the signed part
     (goto-char (point-min))
     (setq syntax3 (mew-decode-singlepart nil nil edit))
     (setq privacy (mew-syntax-get-privacy syntax3))
-    (if (and privacy (not (equal privacy "")))
-	(setq result (concat result "\n\t" privacy)))
-    (mew-syntax-set-privacy syntax3 result)
+    (if privacy (setq result (concat result "\n\t")))
+    (mew-syntax-set-privacy syntax3
+			    (cons (list mew-ct-mls protocol result) privacy))
     ;; Throw away garbage
     (if (file-exists-p file1) (delete-file file1))
     (if (file-exists-p file2) (delete-file file2))
@@ -657,9 +569,7 @@ See also mew-mime-fields."
 (defun mew-save-decode-form (beg end)
   (mew-flet
    (let (syntax file)
-     (if (not (file-exists-p mew-temp-dir))
-	 (mew-make-directory mew-temp-dir)) ;; just in case
-     (setq file (make-temp-name mew-temp-file)) 
+     (setq file (mew-make-temp-name))
      (save-excursion
        (save-restriction
 	 (narrow-to-region beg end)
@@ -667,15 +577,12 @@ See also mew-mime-fields."
 	 (setq syntax (mew-decode-singlepart))
 	 (write-region (mew-syntax-get-begin syntax)
 		       (mew-syntax-get-end syntax)
-		       file)
-	 ))
-     file
-     )
-   ))
+		       file nil 'no-msg)))
+     file)))
 
 (defun mew-decode-crlf-magic ()
   (let ((case-fold-search t)
-	(cte "7bit")
+	(cte mew-7bit)
 	key start match)
     (save-excursion
       (goto-char (point-min))
@@ -687,7 +594,7 @@ See also mew-mime-fields."
 	  (if (string-match "^\r?$" key)
 	      (progn
 		(save-restriction
-		  (if (string-match "binary" cte) ;; cte == "binary^M"
+		  (if (string-match mew-bin cte)
 		      (narrow-to-region (point-min) (1+ start))
 		    (narrow-to-region (point-min) (point-max)))
 		  (goto-char (point-min))
@@ -695,75 +602,108 @@ See also mew-mime-fields."
 		    (replace-match "\n" nil t)))
 		(throw 'header nil)))
 	  (forward-line)
-	  (while (looking-at mew-lwsp) (forward-line))
+	  (mew-header-goto-next)
 	  (setq match (mew-buffer-substring start (1- (point))))
-	  (setq cte (mew-header-syntax match))
-	  )))))
+	  (setq cte (mew-addrstr-parse-value match)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Header arrange in a message buffer
 ;;;
 
-(defun mew-make-field-regex (fields)
-  (mapconcat (function (lambda (x) (concat "^" x))) fields "\\|")
-  )
-
-(defun mew-make-regexes ()
-  (setq mew-field-visible-regex 
-	(mew-make-field-regex mew-field-visible))
-  (setq mew-field-invisible-regex 
-	(mew-make-field-regex mew-field-invisible))
-  (setq mew-field-visible-length
-	(length mew-field-visible))
-  )
-
-(defun mew-header-arrange ()
-  ;; just in case
-  (if (not mew-field-visible-regex)
-      (mew-make-regexes))
-  (goto-char (point-min))
-  (re-search-forward mew-eoh nil t) ;; xxx hack here
+(defun mew-header-arrange (&optional nodisplay)
+  (mew-header-goto-end)
   (if (> (count-lines (point-min) (point)) mew-header-max-length)
       (message "Header is too long")
-    (let* ((len mew-field-visible-length)
-	   (visible-regex mew-field-visible-regex)
-	   (invisible-regex mew-field-invisible-regex)
-	   (visible (make-vector len nil))
-	   (others nil) (key "") (line "") (beg nil) (n nil))
+    (let* ((len (length mew-field-spec))
+	   (visibles (make-vector len nil))
+	   (hilightp (and (or window-system mew-xemacs-p)
+			  mew-use-highlight-header))
+	   (case-fold-search t)
+	   others key beg med n i key-face val-face
+	   n-spec visiblep)
       (save-restriction
 	(narrow-to-region (point-min) (point))
 	(goto-char (point-min))
-	(while (re-search-forward mew-keyval nil t)
-	  (setq key (mew-match 1))
-	  (setq beg (match-beginning 0))
-	  (forward-line)
-	  (while (looking-at mew-lwsp) (forward-line))
-	  (if (string-match invisible-regex key)
+	(while (not (eobp))
+	  (if (not (looking-at mew-keyval))
+	      (forward-line)
+	    (setq key (mew-match 1))
+	    (setq beg (match-beginning 0))
+	    (setq med (match-end 0))
+	    (forward-line)
+	    (mew-header-goto-next)
+	    (setq n-spec (mew-assoc-match3 key mew-field-spec 0))
+	    (setq n (nth 0 n-spec))
+	    (setq visiblep (nth 2 n-spec))
+	    (if hilightp
+		(progn
+		  (setq key-face (or (nth 3 n-spec)
+				     mew-highlight-header-face-key))
+		  (setq val-face (or (nth 4 n-spec)
+				     mew-highlight-header-face-marginal))
+		  (put-text-property beg med 'face key-face)
+		  (put-text-property med (point) 'face val-face)))
+	    (cond
+	     ((null n-spec) ;; others
+	      (setq others (concat others (buffer-substring beg (point))))
+	      (delete-region beg (point)))
+	     (visiblep
+	      (aset visibles n
+		    (concat (aref visibles n)
+			    (buffer-substring beg (point))))
+	      (delete-region beg (point)))
+	     (t ;; invisible
 	      ()
-	    (setq line (mew-buffer-substring beg (point)))
-	    (delete-region beg (point))
-	    (if (null (string-match visible-regex key))
-		(setq others (concat others line))
-	      (setq n (mew-member-match key mew-field-visible))
-	      (aset visible n (concat line (aref visible n)))
 	      ))
-	  ))
-      (if (not mew-field-other-visible)
-	  (and others (insert others)))
-      (recenter 0)
-      (setq n 0)
-      (while (< n len)
-	(if (aref visible n) (insert (aref visible n)))
-	(setq n (1+ n))
-	)
-      (if mew-field-other-visible
-	  (and others (insert others)))
+	    )))
+      (and (not mew-field-other-visible) others (insert others))
+      (or nodisplay (recenter 0))
+      (setq i 0)
+      (while (< i len)
+	(if (aref visibles i) (insert (aref visibles i)))
+	(setq i (1+ i)))
+      (and mew-field-other-visible others (insert others))
       )
-    ;; Some nice features for XEmacs.
-    (if mew-xemacs-p (highlight-headers (point-min) (point-max) nil))
+    (mew-highlight-x-face (point-min) (point))
     )
-  (goto-char (point-max))
-  )
+  ;; when analysis, body will be inserted later.
+  ;; when non-analysis, body is already inserted.
+  ;; Sigh!
+  (mew-header-goto-end)
+  (mew-header-set)
+  (forward-line))
 
 (provide 'mew-decode)
+
+;;; Copyright Notice:
+
+;; Copyright (C) 1996, 1997, 1998 Mew developing team.
+;; All rights reserved.
+
+;; Redistribution and use in source and binary forms, with or without
+;; modification, are permitted provided that the following conditions
+;; are met:
+;; 
+;; 1. Redistributions of source code must retain the above copyright
+;;    notice, this list of conditions and the following disclaimer.
+;; 2. Redistributions in binary form must reproduce the above copyright
+;;    notice, this list of conditions and the following disclaimer in the
+;;    documentation and/or other materials provided with the distribution.
+;; 3. Neither the name of the team nor the names of its contributors
+;;    may be used to endorse or promote products derived from this software
+;;    without specific prior written permission.
+;; 
+;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
+;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+;; PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE TEAM OR CONTRIBUTORS BE
+;; LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+;; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+;; SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+;; BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+;; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
+;; OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
+;; IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+;;; mew-decode.el ends here
