@@ -1,26 +1,20 @@
-;;;			      mew-ext.el
-;;;
-;;;	      Copyright (C) 1996-1997  Kazuhiko Yamamoto
-;;;
-;;;		   This emacs lisp library conforms
-;;;		GNU GENERAL PUBLIC LICENSE Version 2.
-;;;
-;;; Author:  Kazuhiko Yamamoto <kazu@is.aist-nara.ac.jp>
-;;; Created: November 13, 1996
-;;; Revised: April    21, 1997
-;;;
+;;; mew-ext.el --- Message/External-Body support for Mew
 
-(defconst mew-ext-version "mew-ext.el version 0.12")
+;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Created: Nov 13, 1996
+;; Revised: Aug 24, 1998
+
+;;; Code:
+
+(defconst mew-ext-version "mew-ext.el version 0.20")
 
 (require 'mew)
 (eval-when-compile
   (cond
-   ((mew-which "efs.el" load-path)
+   ((mew-which-el "efs" load-path)
     (require 'efs))
-   ((mew-which "ange-ftp.el" load-path)
-    (require 'ange-ftp))
-   )
-  )
+   ((mew-which-el "ange-ftp" load-path)
+    (require 'ange-ftp))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -28,6 +22,18 @@
 ;;;
 
 (defvar mew-ext-prog-url "netscape")
+(defvar mew-ext-prog-url-args '("-install"))
+
+;; If you want to use w3.el instead of "netscape", put the 
+;; following in .emacs.
+;;(setq mew-ext-prog-url "w3")
+;;(setq mew-ext-prog-url-args nil)
+
+;; If you want to use lynx instead of "netscape", put the 
+;; following in .emacs.
+;;(setq mew-ext-prog-url "kterm")
+;;(setq mew-ext-prog-url-args '("-e" "lynx" "-color"))
+
 (defvar mew-ext-anon-ftp-method 'ftp
   "A method to get the message body for access-type=anon-ftp.
 If 'ftp is specified, ange-ftp or efs is used. If 'http is specified,
@@ -40,8 +46,7 @@ w3 is used.")
 (defvar mew-ext-default-access-type "anon-ftp")
 
 (defvar mew-ext-ftp-server-list 
-  '("ftp.aist-nara.ac.jp" "ftp.csce.kyushu-u.ac.jp" "sh.wide.ad.jp")
-  )
+  '("ftp.Mew.org" "sh.wide.ad.jp"))
 
 (defvar mew-ext-encode-switch
   '(("ftp"         . mew-ext-encode-ftp)
@@ -49,8 +54,7 @@ w3 is used.")
     ("anon-ftp"    . mew-ext-encode-anon-ftp)
     ("local-file"  . mew-ext-encode-local-file)
     ("mail-server" . mew-ext-encode-mail-server)
-    ("url"         . mew-ext-encode-url))
-  )
+    ("url"         . mew-ext-encode-url)))
 
 ;;
 ;; decode
@@ -61,74 +65,59 @@ w3 is used.")
     ("tftp"        . mew-ext-tftp)
     ("anon-ftp"    . mew-ext-anon-ftp)
     ("mail-server" . mew-ext-mail-server)
-    ("url"         . mew-ext-url)) ;; RFC2017
-  )
+    ("url"         . mew-ext-url))) ;; RFC2017
 
 (defvar mew-ext-include-switch 
-  '(("local-file" . mew-ext-include-local-file))
-  )
+  '(("local-file" . mew-ext-include-local-file)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; environment variables
 ;;;
 
-(defvar mew-buffer-ext " *mew ext*")
-(defvar mew-ext-suffix ".ext")
-
 (cond
- ((mew-which "efs.el" load-path)
+ ((mew-which-el "efs" load-path)
   (defun mew-ext-file-name-completion (file path)
     (require 'efs)
     (let ((efs-tmp-name-template mew-temp-file))
-      (efs-file-name-completion file path)
-      ))
+      (efs-file-name-completion file path)))
   (defun mew-ext-file-name-all-completions (file path)
     (require 'efs)
     (let ((efs-tmp-name-template mew-temp-file))
-      (efs-file-name-all-completions file path)
-      ))
+      (efs-file-name-all-completions file path)))
   (defun mew-ext-expand-dir (host user dir)
     (require 'efs)
     (let ((efs-tmp-name-template mew-temp-file) exp)
       (setq exp (efs-expand-file-name (format "/%s@%s:%s" user host dir)))
       (if (string-match ".*:\\(.*\\)$" exp)
-	  (mew-match 1 exp))
-      ))
+	  (mew-match 1 exp))))
   (defun mew-ext-copy-file-internal (remote local passwd)
     (require 'efs)
     (let ((efs-tmp-name-template mew-temp-file)
 	  (efs-generate-anonymous-password passwd)
 	  (parsed (efs-ftp-path remote)))
       (efs-copy-file-internal remote parsed local nil 
-			      nil nil nil nil t 'image)
-      ))
-  )
- ((mew-which "ange-ftp.el" load-path)
+			      nil nil nil nil t 'image))))
+ ((mew-which-el "ange-ftp" load-path)
   (defun mew-ext-file-name-completion (file path)
     (require 'ange-ftp)
     (let ((ange-ftp-tmp-name-template mew-temp-file))
-      (ange-ftp-file-name-completion file path)
-      ))
+      (ange-ftp-file-name-completion file path)))
   (defun mew-ext-file-name-all-completions (file path)
     (require 'ange-ftp)
     (let ((ange-ftp-tmp-name-template mew-temp-file))
-      (ange-ftp-file-name-all-completions file path)
-      ))
+      (ange-ftp-file-name-all-completions file path)))
   (defun mew-ext-expand-dir (host user dir)
     (require 'ange-ftp)
     (let ((ange-ftp-tmp-name-template mew-temp-file) exp)
       (setq exp (ange-ftp-expand-file-name (format "/%s@%s:%s" user host dir)))
       (if (string-match ".*:\\(.*\\)$" exp)
-	  (mew-match 1 exp))
-      ))
+	  (mew-match 1 exp))))
   (defun mew-ext-copy-file-internal (remote local passwd)
     (require 'ange-ftp)
     (let ((ange-ftp-tmp-name-template mew-temp-file)
 	  (ange-ftp-generate-anonymous-password passwd))
-      (ange-ftp-copy-file-internal remote local t nil nil nil t)
-      ))
-  )
+      (ange-ftp-copy-file-internal remote local t nil nil nil t))))
  )
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -142,81 +131,79 @@ w3 is used.")
       (message "Can't insert external-body here")
     (let* ((nums (mew-attach-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
-	   (mimedir (mew-expand-file-name (mew-draft-to-mime (buffer-name))))
-	   (file (concat (mew-attach-random-filename) mew-ext-suffix))
-	   (cnt 0) (max (length mew-attach-random-filename))
-	   filepath ct)
+	   (mimedir (mew-expand-folder (mew-draft-to-mime (buffer-name))))
+	   file filepath ct)
       ;; mimedir / {subdir/} dir
       (if (not (equal subdir "")) 
 	  (setq mimedir (expand-file-name subdir mimedir)))
       ;; mimedir / file
-      (setq filepath (expand-file-name file mimedir))
-      (while (and (file-exists-p filepath) (< cnt max))
-	(setq file (concat (mew-attach-random-filename) mew-ext-suffix))
-	(setq filepath (expand-file-name file mimedir))
-	(setq cnt (1+ cnt)))
+      (setq filepath (mew-random-filename mimedir mew-ext-suffix))
       (if (file-exists-p filepath)
 	  (message "Could not make a file for external-body, sorry.")
+	(setq file (file-name-nondirectory filepath))
 	(setq ct (mew-ext-encode filepath))
 	(setq mew-encode-syntax
 	      (mew-syntax-insert-entry
 	       mew-encode-syntax 
 	       nums
 	       (mew-encode-syntax-single file ct)))
-	(mew-encode-syntax-print mew-encode-syntax)
-	))
-    ))
+	(mew-encode-syntax-print mew-encode-syntax)))))
+
+(defun mew-create-content-id ()
+  ;; this is not unique if used with very short interval.
+  ;; but it's ok
+  (format "<%s.%s.%s@%s>" (nth 0 (current-time)) (nth 1 (current-time)) 
+	  (emacs-pid) (system-name)))
 
 (defun mew-ext-encode (filename)
   (let (buf ret access-type ct name)
     (save-excursion
       (set-buffer (get-buffer-create mew-buffer-ext))
       (setq buf (current-buffer))
-      (erase-buffer)
+      (mew-erase-buffer)
       ;;content-header
-      (setq access-type (mew-input "Access type" mew-ext-encode-switch t 
-				   mew-ext-default-access-type))
+      (setq access-type (mew-input-general
+			 "Access type" mew-ext-encode-switch t 
+			 mew-ext-default-access-type))
       (setq ret (funcall (cdr (assoc access-type mew-ext-encode-switch))))
       ;;message-header
       (cond 
-       ((string-equal access-type "url")
+       ((string= access-type "url")
 	(setq name "URL")
 	(setq ct "Text/Html"))
-       ((string-equal access-type "mail-server")
+       ((string= access-type "mail-server")
 	(setq name "Mail server's file")
 	(setq ct mew-ct-apo))
        (t 
 	(setq name (file-name-nondirectory
-		    (mew-header-syntax (mew-syntax-get-member ret "name"))))
+		    (mew-chop (mew-syntax-get-member ret "name"))))
 	;; name is quoted
-	(setq ct (capitalize (mew-file-content
-			      (mew-file-attr name mew-mime-content-type)))))
-       )
+	(if (equal name "")
+	    (setq ct mew-ct-apo)
+	  (setq ct (capitalize (mew-attr-get-ct (mew-attr-by-file name)))))))
       (setq ct (mew-input-type "Type for %s (%s): " name ct
 			       mew-mime-content-type-list))
       (mew-header-insert-here mew-ct: ct)
-      (mew-header-insert-here "Message-ID:" (mew-create-message-id))
+      (mew-header-insert-here mew-cid: (mew-create-content-id))
       (insert "\n")
       (if (not (string= access-type "mail-server"))
 	  () ;; message-body is not necessary
 	;;message-body
 	(insert (read-string "Input message to the mail-server: "))
-	(insert "\n")
-	)
-      (write-file filename)
-      )
+	(insert "\n"))
+      (write-file filename))
     (kill-buffer buf)
-    (cons mew-ct-ext (cons (format "access-type=\"%s\"" access-type) ret))
-    ))
+    (cons mew-ct-ext (cons (format "access-type=\"%s\"" access-type) ret))))
 
 (defun mew-ext-encode-ftp ()
   ;; "name" "site" "directory" "mode"
-  (let ((mew-ext-host (mew-input "FTP server"
-				 (if mew-ext-ftp-server-list 
-				     (mapcar (function list) 
-					     mew-ext-ftp-server-list))
-				 nil
-				 (car mew-ext-ftp-server-list)))
+  (let ((mew-ext-host (mew-input-general 
+		       "FTP server"
+		       (if mew-ext-ftp-server-list 
+			   (mapcar (function list) 
+				   mew-ext-ftp-server-list))
+		       nil
+		       (car mew-ext-ftp-server-list)))
 	mew-ext-user path dir file ret)
     (setq ret (list (format "site=\"%s\"" mew-ext-host)))
     (setq mew-ext-user (read-string (format "User name at %s: " mew-ext-host)
@@ -224,70 +211,62 @@ w3 is used.")
     (setq path (mew-input-rfile "Filename :"))
     (setq file (file-name-nondirectory path))
     (setq dir (file-name-directory path))
-    (if (string-match "~" dir)
+    (if (and dir (string-match mew-home dir))
 	(setq dir (mew-ext-expand-dir mew-ext-host mew-ext-user dir)))
     (cond
      (dir
       (setq ret (cons (format "directory=\"%s\"" dir) ret))
       (setq ret (cons (format "name=\"%s\"" file) ret)))
      (t
-      (setq ret (cons (format "name=\"%s\"" file) ret))
-      )
-     )
-    ret
-    ))
+      (setq ret (cons (format "name=\"%s\"" file) ret))))
+    ret))
 
-(defun mew-ext-encode-tftp () ;; xxx
-  ;; not yet
+(defun mew-ext-encode-tftp ()
+  ;; xxx not yet
   )
 
 (defun mew-ext-encode-anon-ftp ()
   ;; "name" "site" "directory" "mode"
   (let ((mew-ext-user "anonymous")
-	(mew-ext-host (mew-input "FTP server"
-				 (if mew-ext-ftp-server-list 
-				     (mapcar (function list) 
-					     mew-ext-ftp-server-list))
-				 nil
-				 (car mew-ext-ftp-server-list)))
+	(mew-ext-host (mew-input-general
+		       "FTP server"
+		       (if mew-ext-ftp-server-list 
+			   (mapcar (function list) 
+				   mew-ext-ftp-server-list))
+		       nil
+		       (car mew-ext-ftp-server-list)))
 	path dir file ret)
     (setq ret (list (format "site=\"%s\"" mew-ext-host)))
     (setq path (mew-input-rfile "Filename :"))
     (setq file (file-name-nondirectory path))
     (setq dir (file-name-directory path))
-    (if (string-match "~" dir)
+    (if (and dir (string-match mew-home dir))
 	(setq dir (mew-ext-expand-dir mew-ext-host mew-ext-user dir)))
     (cond
      (dir
       (setq ret (cons (format "directory=\"%s\"" dir) ret))
       (setq ret (cons (format "name=\"%s\"" file) ret)))
      (t
-      (setq ret (cons (format "name=\"%s\"" file) ret))
-      )
-     )
-    ret
-    ))
+      (setq ret (cons (format "name=\"%s\"" file) ret))))
+    ret))
 
 (defun mew-ext-encode-local-file ()
   ;; "name" "site"
-  (let ((file (read-file-name "File name: ")))
-    (list (format "name=\"%s\"" (expand-file-name file)))
-    ))
+  (let ((file (mew-input-file-name "File name: ")))
+    (list (format "name=\"%s\"" (expand-file-name file)))))
 
 (defun mew-ext-encode-mail-server ()
   ;; "server" "subject"
   (let (server subject)
     (setq server (mew-input-address "Server address:"))
-    (setq subject (read-string "Subject: "))
+    (setq subject (read-string (concat mew-subj: " ")))
     (list (format "server=\"%s\"" server)
-	  (format "subject=\"%s\"" subject))
-    ))
+	  (format "subject=\"%s\"" subject))))
    
 (defun mew-ext-encode-url ()
   ;; "url"
   (let ((url (read-string "URL: ")))
-    (list (format "url=\"%s\"" url))
-    ))
+    (list (format "url=\"%s\"" url))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -301,24 +280,23 @@ w3 is used.")
 (defun mew-mime-external-body (begin end params &optional execute)
   ;; message-buffer
   (let* ((access-type (mew-syntax-get-member params "access-type"))
-	 (program (cdr (mew-assoc access-type mew-ext-switch 0 t))))
+	 (program (cdr (mew-assoc-match access-type mew-ext-switch 0))))
     ;; xxx expire
     (if (and (symbolp program) (fboundp program))
-	(funcall program begin end params execute))
-    ))
+	(funcall program begin end params execute))))
 
 (defun mew-ext-ftp (begin end params execute)
   (let* ((site (mew-syntax-get-member params "site"))
 	 (directory (mew-syntax-get-member params "directory"))
 	 (name (mew-syntax-get-member params "name"))
 	 (size (mew-syntax-get-member params "size"))
-	 (filepath nil) (localfile nil) (remotefile nil) (getit t)
-	 (username "") (buffer-read-only nil))
+	 (getit t) (username "") (buffer-read-only nil)
+	 filepath localfile lfname remotefile)
     (if directory
-	(if (string-match ".*/$" directory) ;; xxx Win95???
-	    (setq filepath (concat directory name))
-	  (setq filepath (concat directory "/" name)))
+	(setq filepath (concat (file-name-as-directory directory) name))
       (setq filepath name))
+    (if (and filepath (string-match "^[A-Za-z]:/.+" filepath)) ;; drive letter
+	(setq filepath (substring filepath 2 (match-end 0))))
     (erase-buffer)
     (insert " ####### ####### ######  \n"
 	    " #          #    #     # \n"
@@ -328,33 +306,30 @@ w3 is used.")
 	    " #          #    #       \n"
 	    " #          #    #       \n"
             "\n\n")
-    (insert "You can get the message content by FTP\n\n"
-            (format "SITE:\t%s\n" site)
-            (format "FILE:\t%s\n" filepath))
-    (if size (insert (format "SIZE:\t%s bytes\n" size)))
-    (if (not (and execute (mew-y-or-n-p "Get the message? ")))
+    (insert "You can get the message content by FTP\n\n")
+    (mew-insert "SITE:\t%s\n" site)
+    (mew-insert "FILE:\t%s\n" filepath)
+    (mew-insert "SIZE:\t%s bytes\n" size)
+    (if (null execute)
 	(insert "\nTo get this file, type "
 		(substitute-command-keys
 		 "\\<mew-summary-mode-map>\\[mew-summary-execute-external]."))
       (setq username (read-string (format "User name at %s: " site)
 				  (user-login-name)))
       (setq remotefile (format "/%s@%s:%s" username site filepath))
-      (setq localfile
-	    (expand-file-name 
-	     (read-file-name "Save to: " 
-			     (format "%s%s" default-directory name))))
+      (setq localfile (mew-input-file-name 
+		       "Save to: "
+		       (format "%s%s" default-directory name)))
+      (setq lfname (file-name-nondirectory localfile))
       (if (file-exists-p localfile)
-	  (if (mew-y-or-n-p (format "The local file exists. Overwrite? "))
+	  (if (y-or-n-p (format "%s exists. Overwrite? " lfname))
 	      (delete-file localfile)
 	    (setq getit nil)
 	    (message "The file wasn't retrieved")))
-      (if getit (mew-ext-copy-file-internal remotefile localfile nil))
-      )
-    ))
+      (if getit (mew-ext-copy-file-internal remotefile localfile nil)))))
 
 (defun mew-ext-tftp (begin end params execute)
-  (message "access-type TFTP is not supported yet")
-  )
+  (message "access-type TFTP is not supported yet"))
 
 (defun mew-ext-anon-ftp (begin end params execute)
   (let* ((site (mew-syntax-get-member params "site"))
@@ -362,12 +337,12 @@ w3 is used.")
 	 (name (mew-syntax-get-member params "name"))
 	 (size (mew-syntax-get-member params "size"))
 	 (getit t) (buffer-read-only nil)
-	 filepath localfile remotefile url)
+	 filepath localfile lfname remotefile url)
     (if directory
-	(if (string-match ".*/$" directory) ;; xxx Win95???
-	    (setq filepath (concat directory name))
-	  (setq filepath (concat directory "/" name)))
+	(setq filepath (concat (file-name-as-directory directory) name))
       (setq filepath name))
+    (if (and filepath (string-match "^[A-Za-z]:/.+" filepath)) ;; drive letter
+	(setq filepath (substring filepath 2 (match-end 0))))
     (erase-buffer)
     (insert " Anonymous \n"
             " ####### ####### ######  \n"
@@ -378,22 +353,24 @@ w3 is used.")
 	    " #          #    #       \n"
 	    " #          #    #       \n"
             "\n\n")
-    (insert "You can get the message content by FTP\n\n"
-            (format "SITE:\t%s\n" site)
-            (format "FILE:\t%s\n" filepath))
-    (if size (insert (format "SIZE:\t%s bytes\n" size)))
-    (if (null (and execute (mew-y-or-n-p "Get the message? ")))
+    (insert "You can get the message content by FTP\n\n")
+    (mew-insert "SITE:\t%s\n" site)
+    (mew-insert "FILE:\t%s\n" filepath)
+    (mew-insert "SIZE:\t%s bytes\n" size)
+    (if (null execute)
 	(insert "\nTo get this file, type "
 		(substitute-command-keys
 		 "\\<mew-summary-mode-map>\\[mew-summary-execute-external]."))
       (setq remotefile (format "/%s@%s:%s" "anonymous" site filepath))
-      (setq url (format "ftp://%s%s" site filepath))
-      (setq localfile
-	    (expand-file-name 
-	     (read-file-name "Save to: "
-			     (format "%s%s" default-directory name))))
+      (if (string-match "^[/\:]" filepath)
+	  (setq url (format "ftp://%s%s" site filepath))
+	(setq url (format "ftp://%s/%s" site filepath)))
+      (setq localfile (mew-input-file-name 
+		       "Save to: "
+		       (format "%s%s" default-directory name)))
+      (setq lfname (file-name-nondirectory localfile))
       (if (file-exists-p localfile)
-	  (if (mew-y-or-n-p (format "The local file exists. Overwrite? "))
+	  (if (y-or-n-p (format "%s exists. Overwrite? " lfname))
 	      (delete-file localfile)
 	    (setq getit nil)
 	    (message "The file wasn't retrieved"))
@@ -405,11 +382,7 @@ w3 is used.")
 	  (mew-ext-copy-file-internal remotefile localfile mew-mail-address))
 	 ((eq mew-ext-anon-ftp-method 'http)
 	  (require 'w3)
-	  (w3-fetch url))
-	 )
-	)
-      )
-    ))
+	  (w3-fetch url)))))))
 
 (defun mew-ext-mail-server (begin end params execute)
   (let ((server (mew-syntax-get-member params "server"))
@@ -417,19 +390,19 @@ w3 is used.")
 	(size (mew-syntax-get-member params "size"))
 	(start nil)
 	(buffer-read-only nil))
+
     (erase-buffer)
-    (insert "#     #    #      ###   #\n"
-            "##   ##   # #      #    #\n"
-            "# # # #  #   #     #    #\n"
-            "#  #  # #     #    #    #\n"
-            "#     # #######    #    #\n"
-            "#     # #     #    #    #\n"
-            "#     # #     #   ###   #######\n"
+    (insert " #     #    #      ###   #\n"
+            " ##   ##   # #      #    #\n"
+            " # # # #  #   #     #    #\n"
+            " #  #  # #     #    #    #\n"
+            " #     # #######    #    #\n"
+            " #     # #     #    #    #\n"
+            " #     # #     #   ###   #######\n"
             "\n\n")
-    (insert "You can get the message by e-mail\n"
-            "\n"
-            (format "SERVER:\t\t%s\n" server))
-    (if size (insert (format "SIZE:\t%s bytes\n" size)))
+    (insert "You can get the message by e-mail\n\n")
+    (mew-insert "SERVER:\t\t%s\n" server)
+    (mew-insert "SIZE:\t%s bytes\n" size)
     (save-excursion
       (set-buffer (mew-current-get 'cache))
       (save-restriction
@@ -441,18 +414,16 @@ w3 is used.")
 	(setq start (point))
 	))
     ;; pickd up source from 'mew-send
-    (if (null (and execute
-		   (mew-y-or-n-p (format "Send a message to %s? " server))))
+    (if (null execute)
 	(insert "\nTo send this mail, type "
 		(substitute-command-keys
 		 "\\<mew-summary-mode-map>\\[mew-summary-execute-external]."))
-      (mew-summary-send nil server nil subject)
+      (mew-summary-send server nil subject)
+      (goto-char (point-max))
       (insert-buffer-substring (mew-current-get 'cache) start end)
-      (mew-draft-make-mime)
-      (if (mew-y-or-n-p "Send this message? ")
-	  (mew-draft-real-send-letter)
-	))
-    ))
+      (mew-draft-make-message)
+      (if (y-or-n-p "Send this message? ")
+	  (mew-draft-real-send-letter)))))
 
 (defun mew-ext-url (begin end params execute)
   (let ((url (mew-syntax-get-member params "url"))
@@ -461,28 +432,36 @@ w3 is used.")
 	(buffer-read-only nil))
     (erase-buffer)
     (insert "#     # ######  #\n"
-            "#     # #     # #\n"
-            "#     # #     # #\n"
-            "#     # ######  #\n"
-            "#     # #   #   #\n"
-            "#     # #    #  #\n"
-            " #####  #     # #######\n"
-            "\n\n")
-    (insert (format "You can gain access to the url with \"%s\"\n" 
-		    mew-ext-prog-url)
-	    "\n"
-	    (format "URL:\t\t%s\n" url))
-    (if size (insert (format "SIZE:\t%s bytes\n" size)))
-    (if (and execute
-	     (mew-y-or-n-p (format "Gain access to %s ?" url)))
-	(start-process
-	 (format "*mew %s*" mew-ext-prog-url)
-	 mew-buffer-tmp mew-ext-prog-url url)
-      (insert "\nTo show this URL, type "
-	      (substitute-command-keys
-	       "\\<mew-summary-mode-map>\\[mew-summary-execute-external]."))
-      )
-    ))
+	    "#     # #     # #\n"
+	    "#     # #     # #\n"
+	    "#     # ######  #\n"
+	    "#     # #   #   #\n"
+	    "#     # #    #  #\n"
+	    " #####  #     # #######\n"
+	    "\n\n")
+    (mew-insert "URL:\t\t%s\n" url)
+    (mew-insert "SIZE:\t%s bytes\n" size)
+    (insert (format "Browser:\t%s\n"
+		    (cond ((and (symbolp mew-ext-prog-url)
+				(fboundp mew-ext-prog-url))
+			   (symbol-name mew-ext-prog-url))
+			  ((stringp mew-ext-prog-url) mew-ext-prog-url)
+			  (t "none"))))
+    (if (null execute)
+	(insert "\nTo show this URL, type "
+		(substitute-command-keys
+		 "\\<mew-summary-mode-map>\\[mew-summary-execute-external]."))
+      (cond
+       ((and (symbolp mew-ext-prog-url) (fboundp mew-ext-prog-url))
+	(funcall mew-ext-prog-url url))
+       ((equal mew-ext-prog-url "w3")
+	(require 'w3)
+	(w3-fetch-other-frame url))
+       (t
+	(apply (function start-process)
+	       (format "*mew %s*" mew-ext-prog-url)
+	       mew-buffer-tmp mew-ext-prog-url 
+	       (append mew-ext-prog-url-args (list url))))))))
 
 ;;
 ;; include
@@ -490,88 +469,40 @@ w3 is used.")
 
 (defun mew-ext-include-local-file (params)
   (mew-flet
-   (let* ((file (mew-syntax-get-member params "name"))
-	  (size (mew-file-size file)))
+   (let* ((file (mew-syntax-get-member params "name")))
      (if (file-exists-p file)
-	 (if (> size mew-file-max-size)
-	     (if (mew-y-or-n-p
-		  (format "This mail size is %s. Truncate it? " size))
-		 (insert-file-contents file nil 0 mew-file-max-size)
-	       (insert-file-contents file))
-	   (insert-file-contents file)))
-     ))
-  )
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Misc
-;;;
-
-(defun mew-input (prompt alist &optional require-match initial)
-  (let* ((completion-ignore-case t)
-	 (question (if initial (format "%s (%s) : " prompt initial)
-		     (format "(%s) : " prompt)))
-	 (value (completing-read question alist nil require-match nil)))
-    (if (and initial (string= value "")) initial value)
-    ))
-
-(defun mew-create-message-id ()
-  ;; this is not unique if used with very short interval.
-  ;; but it's ok
-  (format "%s.%s.%s@%s" (nth 0 (current-time)) (nth 1 (current-time)) 
-	  (emacs-pid) (system-name))
-  )
-
-(defvar mew-ext-host "")
-(defvar mew-ext-user "")
-(defvar mew-ext-map nil)
-
-(if mew-ext-map
-    ()
-  (setq mew-ext-map (make-sparse-keymap))
-  (define-key mew-ext-map "\t"     'mew-ext-rfile)
-  (define-key mew-ext-map "\r"     'exit-minibuffer)
-  (define-key mew-ext-map "\n"     'exit-minibuffer)
-  (define-key mew-ext-map "\C-g"   'abort-recursive-edit)
-  )
-
-(defun mew-input-rfile (prompt) ;; prompt="To:"
-  (read-from-minibuffer (concat prompt " ") "" mew-ext-map nil)
-  )
-
-(defun mew-ext-rfile ()
-  (interactive)
-  (let (file path comp all)
-    (if (re-search-backward "/" nil t)
-	(forward-char 1)
-      (beginning-of-line))
-    (setq file (buffer-substring (point) (point-max)))
-    (delete-region (point) (point-max))
-    (setq path (format "/%s@%s:%s"
-		       mew-ext-user
-		       mew-ext-host
-		       (buffer-substring (point-min) (point))))
-    (setq comp (mew-ext-file-name-completion file path))
-    (setq all (mew-ext-file-name-all-completions file path))
-    (cond
-     ((eq comp t) 
-      (insert file)
-      (and (get-buffer mew-buffer-completions)
-	   (kill-buffer mew-buffer-completions))
-      )
-     ((equal 1 (length all))
-      (insert (car all))
-      (and (get-buffer mew-buffer-completions)
-	   (kill-buffer mew-buffer-completions))
-      )
-     ((stringp comp)
-      (insert comp)
-      (and (> (length all) 1)
-	   (with-output-to-temp-buffer mew-buffer-completions
-	     (display-completion-list all))))
-     (t (insert file))
-     )
-    )
-  )
+	 (insert-file-contents file)))))
 
 (provide 'mew-ext)
+
+;;; Copyright Notice:
+
+;; Copyright (C) 1996, 1997, 1998 Mew developing team.
+;; All rights reserved.
+
+;; Redistribution and use in source and binary forms, with or without
+;; modification, are permitted provided that the following conditions
+;; are met:
+;; 
+;; 1. Redistributions of source code must retain the above copyright
+;;    notice, this list of conditions and the following disclaimer.
+;; 2. Redistributions in binary form must reproduce the above copyright
+;;    notice, this list of conditions and the following disclaimer in the
+;;    documentation and/or other materials provided with the distribution.
+;; 3. Neither the name of the team nor the names of its contributors
+;;    may be used to endorse or promote products derived from this software
+;;    without specific prior written permission.
+;; 
+;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
+;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+;; PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE TEAM OR CONTRIBUTORS BE
+;; LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+;; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+;; SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+;; BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+;; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
+;; OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
+;; IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+;;; mew-ext.el ends here
